// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files.git)
// DO NOT EDIT

use glib::object::IsA;
use std::fmt;

glib::wrapper! {
    #[doc(alias = "GtkConstraintTarget")]
    pub struct ConstraintTarget(Interface<ffi::GtkConstraintTarget, ffi::GtkConstraintTargetInterface>);

    match fn {
        type_ => || ffi::gtk_constraint_target_get_type(),
    }
}

impl ConstraintTarget {
    pub const NONE: Option<&'static ConstraintTarget> = None;
}

pub trait ConstraintTargetExt: 'static {}

impl<O: IsA<ConstraintTarget>> ConstraintTargetExt for O {}

impl fmt::Display for ConstraintTarget {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        f.write_str("ConstraintTarget")
    }
}
