// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files.git)
// DO NOT EDIT

use crate::Renderer;
#[cfg(any(feature = "v4_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v4_2")))]
use glib::object::Cast;
#[cfg(any(feature = "v4_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v4_2")))]
use glib::translate::*;
use std::fmt;

glib::wrapper! {
    #[doc(alias = "GskGLRenderer")]
    pub struct GLRenderer(Object<ffi::GskGLRenderer, ffi::GskGLRendererClass>) @extends Renderer;

    match fn {
        type_ => || ffi::gsk_gl_renderer_get_type(),
    }
}

impl GLRenderer {
    #[cfg(any(feature = "v4_2", feature = "dox"))]
    #[cfg_attr(feature = "dox", doc(cfg(feature = "v4_2")))]
    #[doc(alias = "gsk_gl_renderer_new")]
    pub fn new() -> GLRenderer {
        assert_initialized_main_thread!();
        unsafe { Renderer::from_glib_full(ffi::gsk_gl_renderer_new()).unsafe_cast() }
    }
}

#[cfg(any(feature = "v4_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v4_2")))]
impl Default for GLRenderer {
    fn default() -> Self {
        Self::new()
    }
}

impl fmt::Display for GLRenderer {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        f.write_str("GLRenderer")
    }
}
