// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files.git)
// DO NOT EDIT

use std::ffi::CStr;

#[doc(alias = "GTK_IM_MODULE_EXTENSION_POINT_NAME")]
pub static IM_MODULE_EXTENSION_POINT_NAME: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_IM_MODULE_EXTENSION_POINT_NAME)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_LEVEL_BAR_OFFSET_FULL")]
pub static LEVEL_BAR_OFFSET_FULL: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_LEVEL_BAR_OFFSET_FULL)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_LEVEL_BAR_OFFSET_HIGH")]
pub static LEVEL_BAR_OFFSET_HIGH: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_LEVEL_BAR_OFFSET_HIGH)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_LEVEL_BAR_OFFSET_LOW")]
pub static LEVEL_BAR_OFFSET_LOW: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_LEVEL_BAR_OFFSET_LOW)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_MEDIA_FILE_EXTENSION_POINT_NAME")]
pub static MEDIA_FILE_EXTENSION_POINT_NAME: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_MEDIA_FILE_EXTENSION_POINT_NAME)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PAPER_NAME_A3")]
pub static PAPER_NAME_A3: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PAPER_NAME_A3).to_str().unwrap()
    });
#[doc(alias = "GTK_PAPER_NAME_A4")]
pub static PAPER_NAME_A4: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PAPER_NAME_A4).to_str().unwrap()
    });
#[doc(alias = "GTK_PAPER_NAME_A5")]
pub static PAPER_NAME_A5: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PAPER_NAME_A5).to_str().unwrap()
    });
#[doc(alias = "GTK_PAPER_NAME_B5")]
pub static PAPER_NAME_B5: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PAPER_NAME_B5).to_str().unwrap()
    });
#[doc(alias = "GTK_PAPER_NAME_EXECUTIVE")]
pub static PAPER_NAME_EXECUTIVE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PAPER_NAME_EXECUTIVE)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PAPER_NAME_LEGAL")]
pub static PAPER_NAME_LEGAL: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PAPER_NAME_LEGAL).to_str().unwrap()
    });
#[doc(alias = "GTK_PAPER_NAME_LETTER")]
pub static PAPER_NAME_LETTER: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PAPER_NAME_LETTER).to_str().unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_COLLATE")]
pub static PRINT_SETTINGS_COLLATE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_COLLATE)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_DEFAULT_SOURCE")]
pub static PRINT_SETTINGS_DEFAULT_SOURCE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_DEFAULT_SOURCE)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_DITHER")]
pub static PRINT_SETTINGS_DITHER: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_DITHER)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_DUPLEX")]
pub static PRINT_SETTINGS_DUPLEX: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_DUPLEX)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_FINISHINGS")]
pub static PRINT_SETTINGS_FINISHINGS: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_FINISHINGS)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_MEDIA_TYPE")]
pub static PRINT_SETTINGS_MEDIA_TYPE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_MEDIA_TYPE)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_NUMBER_UP")]
pub static PRINT_SETTINGS_NUMBER_UP: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_NUMBER_UP)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_NUMBER_UP_LAYOUT")]
pub static PRINT_SETTINGS_NUMBER_UP_LAYOUT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_NUMBER_UP_LAYOUT)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_N_COPIES")]
pub static PRINT_SETTINGS_N_COPIES: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_N_COPIES)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_ORIENTATION")]
pub static PRINT_SETTINGS_ORIENTATION: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_ORIENTATION)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_BASENAME")]
pub static PRINT_SETTINGS_OUTPUT_BASENAME: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_OUTPUT_BASENAME)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_BIN")]
pub static PRINT_SETTINGS_OUTPUT_BIN: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_OUTPUT_BIN)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_DIR")]
pub static PRINT_SETTINGS_OUTPUT_DIR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_OUTPUT_DIR)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_FILE_FORMAT")]
pub static PRINT_SETTINGS_OUTPUT_FILE_FORMAT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_OUTPUT_FILE_FORMAT)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_URI")]
pub static PRINT_SETTINGS_OUTPUT_URI: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_OUTPUT_URI)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_PAGE_RANGES")]
pub static PRINT_SETTINGS_PAGE_RANGES: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_PAGE_RANGES)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_PAGE_SET")]
pub static PRINT_SETTINGS_PAGE_SET: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_PAGE_SET)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_PAPER_FORMAT")]
pub static PRINT_SETTINGS_PAPER_FORMAT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_PAPER_FORMAT)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_PAPER_HEIGHT")]
pub static PRINT_SETTINGS_PAPER_HEIGHT: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_PAPER_HEIGHT)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_PAPER_WIDTH")]
pub static PRINT_SETTINGS_PAPER_WIDTH: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_PAPER_WIDTH)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_PRINTER")]
pub static PRINT_SETTINGS_PRINTER: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_PRINTER)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_PRINTER_LPI")]
pub static PRINT_SETTINGS_PRINTER_LPI: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_PRINTER_LPI)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_PRINT_PAGES")]
pub static PRINT_SETTINGS_PRINT_PAGES: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_PRINT_PAGES)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_QUALITY")]
pub static PRINT_SETTINGS_QUALITY: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_QUALITY)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_RESOLUTION")]
pub static PRINT_SETTINGS_RESOLUTION: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_RESOLUTION)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_RESOLUTION_X")]
pub static PRINT_SETTINGS_RESOLUTION_X: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_RESOLUTION_X)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_RESOLUTION_Y")]
pub static PRINT_SETTINGS_RESOLUTION_Y: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_RESOLUTION_Y)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_REVERSE")]
pub static PRINT_SETTINGS_REVERSE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_REVERSE)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_SCALE")]
pub static PRINT_SETTINGS_SCALE: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_SCALE)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_USE_COLOR")]
pub static PRINT_SETTINGS_USE_COLOR: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_USE_COLOR)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_WIN32_DRIVER_EXTRA")]
pub static PRINT_SETTINGS_WIN32_DRIVER_EXTRA: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_WIN32_DRIVER_EXTRA)
            .to_str()
            .unwrap()
    });
#[doc(alias = "GTK_PRINT_SETTINGS_WIN32_DRIVER_VERSION")]
pub static PRINT_SETTINGS_WIN32_DRIVER_VERSION: once_cell::sync::Lazy<&'static str> =
    once_cell::sync::Lazy::new(|| unsafe {
        CStr::from_ptr(ffi::GTK_PRINT_SETTINGS_WIN32_DRIVER_VERSION)
            .to_str()
            .unwrap()
    });
